<?php

use App\Base\Base;
use App\Exception\custom\GoodsException;
use \Validate\GoodsTccValidate;
use \Validate\GoodsTccIsFinishValidate;
use \Validate\GoodsTccCancelValidate;
use \App\Services\tcc\TccService;


class TccController extends Base
{
    /**
     * 下单商品 tcc try
     * @throws GoodsException
     * @throws \App\Exception\custom\ParamException
     */
    public function place_an_order_tryAction()
    {
        (new GoodsTccValidate())->validate();

        $res = TccService::placeAnOrderTry($this->params["keys"], $this->params["tid"]);
        if ($res) {
            $this->success();
        } else {
            throw new GoodsException(["cus" => 27]);
        }
    }

    /**
     * 下单商品 tcc confirm
     * @throws \App\Exception\custom\ParamException
     * @throws \App\Exception\custom\ParamException
     */
    public function place_an_order_confirmAction()
    {
        (new GoodsTccValidate())->validate();

        $res = TccService::placeAnOrderConfirm($this->params["keys"], $this->params["tid"]);
        if ($res) {
            $this->success();
        } else {
            throw new GoodsException(["cus" => 27]);
        }
    }

    /**
     * 下单商品 tcc cancel
     * @throws \App\Exception\custom\ParamException
     * @throws \App\Exception\custom\ParamException
     */
    public function place_an_order_cancelAction()
    {
        (new GoodsTccValidate())->validate();

        $res = TccService::placeAnOrderCancel($this->params["keys"], $this->params["tid"]);
        if ($res) {
            $this->success();
        } else {
            throw new GoodsException(["cus" => 27]);
        }
    }

    /**
     * 取消订单单 商品tcc try
     * @throws \App\Exception\custom\ParamException
     * @throws \App\Exception\custom\ParamException
     */
    public function cancel_order_tryAction()
    {
        (new GoodsTccValidate())->validate();

        $res = TccService::cancelOrderTry($this->params["keys"], $this->params["tid"]);
        if ($res) {
            $this->success();
        } else {
            throw new GoodsException(["cus" => 27]);
        }
    }

    /**
     * 取消订单单 商品tcc confirm
     * @throws \App\Exception\custom\ParamException
     * @throws \App\Exception\custom\ParamException
     */
    public function cancel_order_confirmAction()
    {
        (new GoodsTccCancelValidate())->validate();

        $res = TccService::cancelOrderConfirm($this->params["keys"], $this->params["tid"], $this->params["type"]);
        if ($res) {
            $this->success();
        } else {
            throw new GoodsException(["cus" => 27]);
        }
    }

    /**
     * 取消订单单 商品tcc cancel
     * @throws \App\Exception\custom\ParamException
     * @throws \App\Exception\custom\ParamException
     */
    public function cancel_order_cancelAction()
    {
        (new GoodsTccValidate())->validate();

        $res = TccService::cancelOrderCancel($this->params["keys"], $this->params["tid"]);
        if ($res) {
            $this->success();
        } else {
            throw new GoodsException(["cus" => 27]);
        }
    }

    /**
     * 判断当前tcc是否执行完成
     * @throws \App\Exception\custom\ParamException
     */
    public function tcc_is_finishAction()
    {
        (new GoodsTccIsFinishValidate())->validate();

        $res = TccService::tccIsFinish($this->params["goods_sku_id_str"], $this->params["tid"]);
        $this->success(["result"=>$res]);
    }
}